package com.javaxing.stock.service.impl;

import io.seata.core.context.RootContext;
import io.seata.rm.tcc.api.BusinessActionContext;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import com.javaxing.stock.entity.StockTbl;
import com.javaxing.stock.mapper.StockTblMapper;
import com.javaxing.stock.service.StockTblService;
import org.springframework.transaction.annotation.Transactional;

@Service
@Slf4j
public class StockTblServiceImpl implements StockTblService{
    @Resource
    private StockTblMapper stockTblMapper;
    /**
     * Try: 库存-扣减数量
     *
     * 定义两阶段提交，在try阶段通过@TwoPhaseBusinessAction注解定义了分支事务的 resourceId，commit和 cancel 方法
     *  name = 该tcc的bean名称,全局唯一
     *  commitMethod = commit 为二阶段确认方法
     *  rollbackMethod = rollback 为二阶段取消方法
     *  BusinessActionContextParameter注解 传递参数到二阶段中
     *
     * @param commodityCode 商品编号
     * @param count 扣减数量
     * @return
     */

    @Transactional
    @Override
    public void deduct(String commodityCode, int count) {
        log.info("当前 XID: {}", RootContext.getXID());

        // 检查库存是否足够扣减，如果不够的话就抛出异常
        StockTbl stockTbl = stockTblMapper.findFirstByCommodityCode(commodityCode);
        if(stockTbl.getCount() < count){
            throw new RuntimeException("库存不足");
        }

        // 开始扣减库存
        stockTblMapper.deductCount(commodityCode,count);
        log.info("库存扣减成功，产品编号：{}，扣减数量：{}",commodityCode,count);
    }

    @Override
    public boolean commit(BusinessActionContext actionContext) {
        log.info("库存commit成功");
        return true;
    }

    @Override
    public boolean rollback(BusinessActionContext actionContext) {
        String commodityCode = actionContext.getActionContext("commodityCode").toString();
        int count = (int) actionContext.getActionContext("count");

        // 回滚库存数量
        stockTblMapper.addCount(commodityCode,count);
        log.info("库存回滚成功");
        return true;
    }
}
